/*********************************************************************************************************************************
  GSMBand_Generic.h
  
  For ESP8266, ESP32, SAMD21/SAMD51, nRF52, SAM DUE, Teensy and STM32 with GSM modules
  
  GSM_Generic is a library for the ESP8266, ESP32, SAMD21/SAMD51, nRF52, SAM DUE, Teensy and STM32 with GSM modules
  
  Based on and modified from MKRGSM Library (https://github.com/arduino-libraries/MKRGSM)
  
  Built by Khoi Hoang https://github.com/khoih-prog/GSM_Generic
  Licensed under GNU Lesser General Public License
  
  Copyright (C) 2017  Arduino AG (http://www.arduino.cc/)
  This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
  as published by the Free Software Foundation, either version 2.1 of the License, or (at your option) any later version.
  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
  You should have received a copy of the GNU General Public License along with this program.
  If not, see <https://www.gnu.org/licenses/>.  
 
  Version: 1.2.4
  
  Version Modified By   Date      Comments
  ------- -----------  ---------- -----------
  1.2.4    K Hoang     11/03/2021 Initial public release to add support to many boards / modules besides MKRGSM 1400 / SARA U201
 **********************************************************************************************************************************/

#pragma once

#ifndef _GSM_BAND_H_INCLUDED
#define _GSM_BAND_H_INCLUDED

#include "GSM_Generic_Debug.h"

#include "GSM_Generic.h"

#include "GSM_Type_Generic.h"

class GSMBand 
{
  public:

    /** Constructor
        @param trace    If true, dumps all AT dialogue to Serial
    */
    GSMBand(bool trace = false);

    /** Forces modem hardware restart, so we begin from scratch
        @param baud      baudRate for SerialGSM to communicate to modem
        @return always returns IDLE status
    */
    GSM3_NetworkStatus_t begin(unsigned long baud = 115200);

    /** Get current modem work band
        @return current modem work band
    */
    String getBand();

    /** Changes the modem operating band
        @param band     Desired new band
        @return true if success, false otherwise
    */
    bool setBand(String band);

  private:

    /** Change the Radio access technology
        @param act   desired access technology
        @return true if success, false otherwise
    */
    bool setRAT(const char* act);
};

#include "GSMBand_Generic_Impl.hpp"

#endif    // _GSM_BAND_H_INCLUDED
